--************************************************************************************************
-- script : UMP failover
-- author : Nimsoft (Rowan Collis) - June 2012
-- contributors : Nimsoft (James Christensen, Mike Ellerbe) - July - September 2012
-- version: 8.51 
-- desc   : this script should run on the secondary nas which is also running the HA probe.
--          When the HA probe detects a failover, an alarm is raised "initiating failover"
--          which triggers this script. It should also run on the primary for failback, triggered
--          by the alarm "stopping failover"
--************************************************************************************************
-- Version  |  Details
--************************************************************************************************
-- 1.0      |  Initial Version
--************************************************************************************************
-- 2.0		| Significant re-write to add and change functionality according to customer requests
--				1. Add a print for the loop counter in the check of the robot (check_umprobot() )
--				2. Add the ability to read in configuration information from an external file.
--				   The file will contain parameter values in the form of key = value pairs.
--				   Example:
--					PriHubAddr = /NMS/Center_Hub/
--					SecHubAddr = /NMS/Center_Hub_HA/
--					PriHub = xdprh01
--					SecHub = xdprh02
--					umpname = xdump01,xdump02
--					dap = 7
--					dashboard_engine = 10
--					wasp = 20
--					robot = 10
--				3. Edit all the configuration files for the UMP probes first, then go back and restart them.
--					do not perform activatation checks until all resart commands have been issued. the order of 
--					execution is as follows:
--					a. check for UMP robot active
--					b. For each UMP robot (1st loop)
--						1. reconfigure UMP probes
--						2. issue _restart commands
--					c. For each UMP robot (2nd loop)
--						1. check each UMP probe for activation
--					d. Issue failover complete or failed alarm message.
--************************************************************************************************
-- 2.2			| Minor edits and fixes resulting from internal Nimsoft testing
--************************************************************************************************
-- 2.3			| Added an explicit amount of time to wait for the nimbus.request - probe_config_get command
--				  to complete when re-configuring the dashboard_engine, dap, and wasp probes. 
--				  Net One testing revealed times when the wasp and dap probes would be re-configured, 
--				  but the dashboard_engine probe_config_get call would recieve a return code of 2 - communication error.
--				  Also put the initial probe_config_get request into a repeat...until loop so that 
--				  a miximum of three attempts will be made with a 60 second pause in between each attempt.
--************************************************************************************************	
-- 2.4			| 8/14/2012 James Christensen - Nimsoft LLC.
--				  Added NMS alarm functions for a failure / error points in the script to augment
--				  the print statements that write to the nas log.
--				  Altered the pathing to the input file per request. Added a check for a successful
--				  of the input file. If it is not successfully read (input buffer is nil) then the 
--				  script will abort/exit with a return code of -1	
--*************************************************************************************************
-- 	2.5		| 9/14/2012 James Christensen - Nimsoft LLC.
--				  function reconfigure _wasp() - Removed the portion of code that were reconfiguring custom 
--				  settings in the individual webapp section of the wasp configuration. Left comment markers 
--				  where the sections of code were removed. these were removed because custom settings in the 
--				  individual webapps is not needed and they were redundant to the ump_common and setup configurations. 
--				  Please see Support Case #87801 - UMP documentation defect
--
--				 function restart_ump_probes() - changed this function to utilize an array that contains the names 
--			     of the ump probes and added a for loop to iterate through the array altering the PDS arguments
--				 for the nimbus.request() probe_deavtive and probe_activate commands instead of the 
--				 linear processing with redundant code for each probe.
--*************************************************************************************************  
---- 	2.5.1		| 10/3/2012 Michael Ellerbe - Nimsoft LLC.
--				  Minor syntax corrections.
--*************************************************************************************************  
-- 	3.0 		| 10/4/2012 Michael Ellerbe - Nimsoft LLC. 
--				  Updating script to include new wasp keys for the UMP v6 release. 
--*************************************************************************************************  
--	3.11			| 12/4/2012 James Christensen - CA Nimsoft LLC
--				  Updated script adding the relationshipviewer webapp custom key (GrahpServiceHost) value change
--				  to the wasp reconfiguration function. The value changes to the secondary hub IP address as provided
--				  in the inputfile.txt on FAILOVER and back to the primary hub server IP address on FAILBACK. This is per
--				  NetOne request and Support case number 93913
--*************************************************************************************************
-- 4.0			| 6/10/2013 James Christensen - CA Nimsoft
--				Updated the script to account for changes for NM 6.5. Added the re-configuration of the service_host
--				key in the ump_common section of the wasp configuration. Moved the discovery_server re-configuration from
--				main setup section to the ump_common section. Added the activation/deactivation of the nis_bridge on
--				the secondary nas probe as the last action the sctip takes (Activate on FAILOVER; deactivate on FAILBACK)
--*****************************************************************************************************
-- 4.01			| 6/17/2013 James Christensen - CA Nimsoft
--				Updated the script portion for activating/de-activating the nis_bridge key of the secondary nas.
--				Additional print messages to the nas.log
--				Added a configuration get and a verification print after a successfull setting of the configuration (rc = 0).
--				Placed the configuration set into a loop which retry the probe_config_set command up to 3
--				times or until rc == 0.
--*******************************************************************************************************
-- 4.02			| 6/18/2013 James Christensen - CA Nimsoft
--				Corrected all occurences of typo for the nimbus.alarm function - changing nimus to nimbus
--				Corrected the Nimsoft address for the Secondary controller and nas proces - removed the double slash (//)
--				Added one more print statement right before the nas restart
--				Fixed variable reference for the dashboard_engine and wasp reconfiguration functions from count to counter
--********************************************************************************************************
-- 4.03			| 7/22/2013 James Christensen - CA Nimsoft
--				Moved the NiS Bridge configuration and nas restart to a separate function - configurNiSBridge ().
--				Commented out the call to execute the configureNiSBridge function at the end of the main body of code
--				per NetOne request.
--**********************************************************************************************************
-- 4.04			| 7/29/2013 James Christensen - CA Nimsoft
--				Added code in the Main section for FAILOVER mose that restarts the service_host probe
--				after the UMP probes have been re-configured and restarted,per NetOne request
--**********************************************************************************************************
-- 4.05			| 8/02/2013 James christensen - CA Nimsoft
--				Altered the code block added in version 4.04 to request that the controller probe deactivate
--				and then activate the service_host probe rather than using the _restart command directly on
--				the probe. Changed this code because in the event the service_host probe is in an error (red)
--				condition, it will not respond to a request to execute callback commands.
--***********************************************************************************************************
-- 4.06			| 8/22/2013 James Christensen - CA Nimsoft
--				NetOne testing showed that the service_host deactivate/activate was occurring twice on failover
--				This was due to the code block being inside the loop for re-configuring the UMP servers.
--				Moved the code block outside the loop between the reconfigure and the check to see if the 
--				probes are up. This is only in the FAILOVER section.
--				This section was also evaluating the success erroneously and sending an alarm and logging
--				that the deactivation and activation were not successfull even though they were.
--				Corrected.
--************************************************************************************************************
-- 4.07			| 7/03/2014 Kazuomi Suzuta - JBST
--				Updated the script to account for changes for NM 7.5. Added the re-configuration of the mpse
--				key and maintenance_mode key in the ump_common section of the wasp configuration.
--************************************************************************************************************
-- 4.08			| 7/28/2015 Kazuomi Suzuta - JBST
--				Updated the script to account for changes for NM 8.2. Added the re-configuration of the ugs
--				key and udm_manager key in the ump_common section of the wasp configuration.
--************************************************************************************************************
-- 4.09			1/08/2016 Rowan Collis - CA
--				Updated the script for 8.31. Remove reconfigure of DAP and Dashboard-engine throughout
--				
--************************************************************************************************************
-- 4.10			6/21/2016 Rowan Collis - CA
--				Updated the script for 8.4. Add reconfigure of trellis key in wasp
--				
--************************************************************************************************************
-- 8.51      	4/04/2017 Rowan Collis - CA
--				Updated the script for 8.51. Add reconfigure of ems key in wasp
--				Support CABI wasp
--                      Add cabirobotname key to inputfile (also need to put CABI robot name in UMP list)
--				
--************************************************************************************************************


-------------------------------------
--  OVERRIDES FOR TESTING
------------------------------------- 
--message1 = "Initiating failover"
--message1 = "Stopping failover,"
-------------------------------------

--************************************************************************************************
--Setup alarm to initiate failover
--The driving nas profile needs to pick up only two HA alarms ie the "Initiating failover" 
--Use "on overdue age" and Probe = HA and message = /Initiating failover.*/ (for secondary nas profile) 
--or /Stopping failover.*/ (for primary nas profile)
--and nms robot name = robotname (of where HA probe is running)
--************************************************************************************************

a=alarm.get()
msg=split(a.message)
message1=msg[1].." "..msg[2]
print(message1)


--************************************************************************************************
-- Setup UMP robots (maybe more than one) and Primary and Secondary hubs
--   (If your UMP robots are attached to the hub which has just gone down
--    then you have to assume that they're jumping to their secondary 
--************************************************************************************************
-- Global variables
PriHubAddr = ""
SecHubAddr = ""
PriHub = ""
SecHub = ""
PriIP = ""
SecIP = ""
umpname = {}
cabirobotname = ""
--  if the nis bridge is required to be turned on and off on the secondary then this is set to "yes" on 
--  on the secondary nas script and "no" on the primary nas script. it will only reconfigure the 
--  secondary nas. This is a variable in the inputfile
--  This does mean it will restart the secondary nas in failover mode so this AO will always appear 
--  to have failed.
nis_bridge_activate = "yes"
-- default number of retry attempts when confirming UMP probes are back up
dap_retries = 10
dashboard_engine_retries = 10
wasp_retries = 10
robot_retries = 10
mode = nil

-- Read data from external file into buffer, get number of bytes read
fname = "C:\\Program Files (x86)\\Nimsoft\\probes\\service\\nas\\conf\\inputfile.txt"
b,nread = file.read (fname)
if b == nil then
	nimbus.alarm(NIML_MINOR,"Unable to get required data from input file: "..fname..". Aborting UMP reconfiguration","UMP_failover")
      printf("exit on fname:%s",fname)
	exit(-1)
end
--printf ("%d bytes read - buffer\n%s",nread,b)

-- Parse out the data from the input buffer
tmp = split(b,"\r")
for i=1,#tmp do
--printf("Line %d\r",i)
--print(tmp[i])
	myTmp = split(tmp[i])
	if substr(tmp[i],"PriHubAddr") then
		PriHubAddr = myTmp[3]
	elseif substr(tmp[i],"SecHubAddr") then
		SecHubAddr = myTmp[3]
	elseif substr(tmp[i],"PriHub") then
		PriHub = myTmp[3]
	elseif substr(tmp[i],"SecHub") then
		SecHub = myTmp[3]
	elseif substr(tmp[i],"PriIP") then
		PriIP = myTmp[3]
	elseif substr(tmp[i],"SecIP") then
		SecIP = myTmp[3]
	elseif substr(tmp[i],"umpname") then
		local ump=split(myTmp[3],",")
		for n=1,#ump do
			umpname[n] = ump[n]
			-- just in case there is a leading space
			umpname[n] = string.gsub(umpname[n]," ","",1)
		end
	elseif substr(tmp[i],"cabirobotname") then
		cabirobotname = myTmp[3]
      elseif substr(tmp[i],"dap") then
		dap_retries = tonumber(myTmp[3])
	elseif substr(tmp[i],"dashboard_engine") then
		dashboard_engine_retries = tonumber(myTmp[3])
	elseif substr(tmp[i],"wasp") then
		wasp_retries = tonumber(myTmp[3])
	elseif substr(tmp[i],"robot") then
		robot_retries = tonumber(myTmp[3])
	end	
end
printf("Primary addr = %s\nSec addr = %s\nPri robot = %s\nSec robot = %s\n",PriHubAddr, SecHubAddr, PriHub, SecHub)
nimbus.alarm(NIML_INFORMATION,"Primary Addr: "..PriHubAddr.." Sec Addr: "..SecHubAddr.." Pri robot: "..PriHub.." Sec robot: "..SecHub,"UMP_Failover")
printf("Retries are: dap - %d; dashboard_engine - %d; wasp - %d\n",dap_retries,dashboard_engine_retries,wasp_retries)

if message1 == "Initiating failover" then
      mode = "FAILOVER"
      nis_bridge_activate = "yes"
      nimbus.alarm(NIML_INFORMATION,"Mode is FAILOVER","UMP_Failover")
elseif message1 == "Stopping failover," then
      mode = "FAILBACK"
      nis_bridge_activate = "no"
      nimbus.alarm(NIML_INFORMATION,"Mode is FAILBACK","UMP_Failover")
else
      print("UMP Failover script Error - incorrect trigger alarm, check NAS profile")
      nimbus.alarm(NIML_WARNING,"UMP Failover script Error - incorrect trigger alarm, check NAS profile","UMP_Failover")
      mode = nil
      nis_bridge_activate = nil
end


--**********************************************************************************************
--  function for UMP robots check - need to wait for the robots to failover before anything
--  (presuming they are attached to primary and secondary hubs and not another hub)
--  you might need to increase count to greater than 6 if your robot failover is a bit slow...
--**********************************************************************************************
function check_umprobot(hubaddr,ump,retries)
   local robotsup = false
   local count=0
   repeat
      res,rc = nimbus.request (hubaddr..ump.."/controller","get_info")
      print(hubaddr..ump.."/controller")
      if rc == 0 then
         robotsup = true
         print(hubaddr..ump.." has jumped, looped "..count.." times")
      else
		 print("Check number "..count.."for "..hubaddr..ump..". Continue checking....")
         sleep (20000)
         count=count+1
      end
   until robotsup or count==retries
   return robotsup
end

---**********************************************************************************************
--             function to restart the ump probes
--			9/14/2012 - James Christensen, Nimsoft LLC.
--			changed this function to utilize an array that contains the names of the ump probes
--			and added a for loop to iterate through the array altering the PDS arguments
--			for the nimbus.request() probe_deavtive and probe_activate commands instead of the 
--			linear processing with redundant code for each probe.
--  4.09	1/8/2016 - Rowan Collis, CA
--			remove the dap and dashboard_engine
--***********************************************************************************************
function restart_ump_probes(hubaddr,ump)
                local err = 0
                local suppkey = "probe_restart"
--4.09                ump_probes = {"dap","dashboard_engine","wasp"}
                ump_probes = {"wasp"}
                args = pds.create()
            
	for _, probe in pairs(ump_probes) do
		print("Restart the ".. probe.." probe")
    	pds.putString (args,"name",probe)
                
		res,rc = nimbus.request (hubaddr..ump.."/controller","probe_deactivate",args,60)
		if rc == 0 then
			print(hubaddr..ump.." "..probe.." deactivate command issued successfully")
		else
			err = rc
			print(hubaddr..ump.." "..probe.." deactivate command error: return code: "..tostring(rc))
			nimbus.alarm(NIML_MINOR,hubaddr..ump.." "..probe.." deactivate command error: return code: "..tostring(rc),suppkey)
		end
		sleep(10000) -- wait 10 seconds, this can be adjusted
		res,rc = nimbus.request (hubaddr..ump.."/controller","probe_activate",args,60)
		if rc == 0 then
			print(hubaddr..ump.." "..probe.." activate command issued successfully")
		else
			err = rc
			print(hubaddr..ump.." "..probe.." activate command error: return code: "..tostring(rc))
			nimbus.alarm(NIML_MINOR,hubaddr..ump.." "..probe.." activate command error: return code: "..tostring(rc),suppkey)
		end
	end
                              
	pds.delete(args)
	return err
end

--***********************************************************************************************
--		Test for probe start up
--***********************************************************************************************
function check_probe(hubaddr,ump,probe,retries)
	print("Testing "..probe.." on "..ump)
	local probeup = false
	local count = 0
	local suppkey = "probe_check-"..ump
	if probe =="dap" then
		cmd = "get_dbs"
	elseif probe == "dashboard_engine" then
		cmd = "GetAcls"
	else
		cmd = "get_info"
	end
	
	repeat
		print(probe.." Check number: "..tostring(count))
		res, rc = nimbus.request (hubaddr..ump.."/"..probe,cmd)
		if rc == 0 then
			probeup = true
			print(hubaddr..ump.."/"..probe.." is now active")
			nimbus.alarm(NIML_INFORMATION,hubaddr..ump.."/"..probe.." is now active",suppkey)
		else
			sleep(20000)
			count=count+1
		end	
	until probeup or count==retries
	return probeup
end
 
	
function reconfigure_wasp(hubaddr,hubname,ump,ip)
   print("Reconfiguring WASP on "..hubaddr..ump)
   local counter = 1
   local rc
   local cfg = {}
   local suppkey = "wasp_config"
   wasppds = pds.create()
   pds.putString(wasppds,"name","wasp")

   args = pds.create()
   pds.putString (args,"name","wasp")
   
	-- 8/9/12 : Added an arbitrary wait time for the probe_config_get command to complete
   	-- 8/13/12 : Added repeat...until loop with a loop counter to make three attempts to get the probe configuration
   repeat
		print("Attempt number "..counter.. "to get WASP probe config.....")
   		cfg,rc = nimbus.request (hubaddr..ump.."/controller","probe_config_get",args,60)
   		if rc == 0 then
   			print("Get WASP configuration successfull....")
   		else
   			sleep(60)
   			counter = counter + 1
   		end
   	until rc == 0 or counter >=4
	
 -- 8.51 CABI wasp passes through this section but doesn't have ump_common section in wasp config so should not try to reconfigure it.  Does have setup section which needs the same reconfigure as ump wasps.
   if rc == 0 then
      waspsetup = cfg["/setup"]
      if waspsetup ~= nil then
         printf("Changing data_engine from = %s",waspsetup.data_engine)

-- data engine
         pds.putString(wasppds,"section","/setup")
         pds.putString(wasppds,"key","data_engine")
         pds.putString(wasppds,"value",hubaddr..hubname.."/data_engine")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(data engine) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(data engine) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end

      else
         print("Missing Wasp Config in Setup section on"..hubaddr..ump)
         nimbus.alarm(NIML_MINOR,"Missing Wasp Config in Setup section on"..hubaddr..ump,suppkey)
      end

      waspcommon = cfg["/ump_common"]
      if waspcommon ~= nil then
         printf("Changing nas from        = %s",waspcommon.nas)
         printf("Changing sla_engine from = %s",waspcommon.sla_engine)
         printf("Changing ace from        = %s",waspcommon.ace)
--4.09         printf("Changing nis_server from = %s",waspcommon.nis_server)
         printf("Changing service_host from = %s",waspcommon.service_host)
         printf("Changing discovery_server from = %s",waspcommon.discovery_server)
         printf("Changing automated_deployment_engine from = %s",waspcommon.automated_deployment_engine)
         printf("Changing maintenance_mode from = %s",waspcommon.maintenance_mode)
         printf("Changing mpse from = %s",waspcommon.mpse)
         printf("Changing ugs from = %s",waspcommon.ugs)
         printf("Changing udm_manager from = %s",waspcommon.udm_manager)

-- nas
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","nas")
         pds.putString(wasppds,"value",hubaddr..hubname.."/nas")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(nas) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(nas) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
      
-- sla_engine
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","sla_engine")
         pds.putString(wasppds,"value",hubaddr..hubname.."/sla_engine")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(sla_engine) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(sla_engine) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end

      
-- ace
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","ace")
         pds.putString(wasppds,"value",hubaddr..hubname.."/ace")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(ace) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(ace) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end

-- automated_deployment_engine
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","automated_deployment_engine")
         pds.putString(wasppds,"value",hubaddr..hubname.."/automated_deployment_engine")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(automated_deployment_engine) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(automated_deployment_engine) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
      
-- nis_server
--4.09         pds.putString(wasppds,"section","/ump_common")
--4.09         pds.putString(wasppds,"key","nis_server")
--4.09         pds.putString(wasppds,"value",hubaddr..hubname.."/nis_server")   
--4.09         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
--4.09         if rc ~= 0 then
--4.09            print("Nimbus Request Error - Probe Config Set(nis_server) for wasp on "..hubaddr..ump.." (error "..rc..")")
--4.09            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(nis_server) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
--4.09         end
         
-- service_host - not present in 8.51+ but is still in ump_common of wasp - remove in future!!
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","service_host")
         pds.putString(wasppds,"value",hubaddr..hubname.."/service_host")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(service_host) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(service_host) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
         
-- discovery_server
         pds.putString(wasppds,"section","/ump_common")
		 pds.putString(wasppds,"key","discovery_server")
      	 pds.putString(wasppds,"value",hubaddr..hubname.."/discovery_server")  
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(discovery_server) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(discovery_server) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
         
-- maintenance_mode
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","maintenance_mode")
         pds.putString(wasppds,"value",hubaddr..hubname.."/maintenance_mode")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(maintenance_mode) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(maintenance_mode) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
         
-- mpse
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","mpse")
         pds.putString(wasppds,"value",hubaddr..hubname.."/mpse")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(mpse) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(mpse) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
         
-- ugs
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","ugs")
         pds.putString(wasppds,"value",hubaddr..hubname.."/ugs")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(ugs) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(ugs) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
         
-- udm_manager
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","udm_manager")
         pds.putString(wasppds,"value",hubaddr..hubname.."/udm_manager")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(udm_manager) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(udm_manager) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
--4.10 
-- trellis
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","trellis")
         pds.putString(wasppds,"value",hubaddr..hubname.."/trellis")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(trellis) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(trellis) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
-- 4.10                 
 --8.51
-- ems
         pds.putString(wasppds,"section","/ump_common")
         pds.putString(wasppds,"key","ems")
         pds.putString(wasppds,"value",hubaddr..hubname.."/ems")   
         result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
         if rc ~= 0 then
            print("Nimbus Request Error - Probe Config Set(ems) for wasp on "..hubaddr..ump.." (error "..rc..")")
            nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(ems) for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
         end
-- 8.51    

        
      else
	  
-- 8.51 - only alarm if the wasp is non CABI
         if ump == cabirobotname then
            print("CABI wasp - No UMP_common section on"..hubaddr..ump)
         else
            print("Missing Wasp Config - UMP_common section on"..hubaddr..ump)
            nimbus.alarm(NIML_MINOR,"ump-failover: missing UMP_common section on"..hubaddr..ump,suppkey)
         end
      end
       
-- check config is set
      cfg,rc = nimbus.request (hubaddr..ump.."/controller","probe_config_get",args)
      if rc == 0 then
         waspcommon = cfg["/ump_common"]
         if waspcommon ~= nil then
            printf("Changed nas to        = %s",waspcommon.nas)
            printf("Changed sla_engine to = %s",waspcommon.sla_engine)
            printf("Changed ace to        = %s",waspcommon.ace)
--4.09            printf("Changed nis_server to = %s",waspcommon.nis_server)
            printf("Changed service_host to = %s",waspcommon.service_host)
            printf("Changed discovery_server to = %s",waspcommon.discovery_server)
            printf("Changed automated_deployment_engine to = %s",waspcommon.automated_deployment_engine)
            printf("Changed maintenance_mode to = %s",waspcommon.maintenance_mode)
            printf("Changed mpse to = %s",waspcommon.mpse)
            printf("Changed ugs to = %s",waspcommon.ugs)
            printf("Changed udm_manager to = %s",waspcommon.udm_manager)
            printf("Changed trellis to = %s",waspcommon.trellis)
		printf("Changed ems to = %s",waspcommon.ems)  --8.51

         else
-- 8.51 - only alarm if non CABI wasp. CABI wasp doesn't have ump common section
            if ump == cabirobotname then
                  print("CABI wasp - No UMP_common section on"..hubaddr..ump)
            else
			print("Missing Wasp Config2 - UMP_common section on"..hubaddr..ump)
			nimbus.alarm(NIML_MINOR,"ump-failover: missing UMP_common section on"..hubaddr..ump,suppkey)
            end
         end
      else
         print("Nimbus Request Error2 - Probe Config Get for wasp on "..hubaddr..ump.." (error "..rc..")")
         nimbus.alarm(NIML_MINOR,"Nimbus Request Error2 - Probe Config Get for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
      end

		      
		-- The reconfiguring for Custom entries in the individual webapps should NOT be required. All should be using the umpcommon entries
		-- for the nas, ace, and dap probes. Removing this part in the script - see case 87801.

		-- Alarm Console Custom NAS: removed
 
		-- Custom Dashboards Custom NAS: removed

		-- Dynamic Views Custom NAS: removed

		-- Dashboard Designer Custom DAP
		-- This is the end of the section from which the custom webapp configuration settings were removed
		-- Primary Hub(10.44.208.24)
		--Secondary Hub(10.44.208.25)
		
		-- Per request  and Support Case number 93913, adding in a custom setting for the Relationship viewer webapp --- 12/3/2012
		-- Relationship viewer Custom ggraphics server
		wasprelationshipviewer = cfg["/webapps/relationshipviewer/custom/uncrypted"]
		if wasprelationshipviewer ~= nil then
        	printf("Changing RelationshipViewer graph server host from = %s",wasprelationshipviewer.GraphServiceHost)
        	pds.putString(wasppds,"section","/webapps/relationshipviewer/custom/uncrypted")
        	pds.putString(wasppds,"key","GraphServiceHost")
        	pds.putString(wasppds,"value",ip)   
        	result,rc = nimbus.request(hubaddr..ump.."/controller","probe_config_set",wasppds)
        	   if rc ~= 0 then
            	print("Nimbus Request Error - Probe Config Set(GraphServiceHost custom) for wasp on "..hubaddr..ump.." (error "..rc..")")
         	   end
      	else
-- 8.51
		    if ump == cabirobotname then
               print("CABI wasp - No Relationship Viewer section on"..hubaddr..ump)
            else
         	   print("Missing Wasp Config in webapps/relationshipviewer/custom/uncrypted section on"..hubaddr..ump)
			end
      	end
		
   else
      print("Nimbus Request Error1 - Probe Config Get for wasp on "..hubaddr..ump.." (error "..rc..")")
      nimbus.alarm(NIML_MINOR,"Nimbus Request Error1 - Probe Config Get for wasp on "..hubaddr..ump.." (error "..rc..")",suppkey)
   end
end

--************************************************************************************************
-- this function will activate or deactivate the NiS Bridge on the secondary (HA) hub nas probe
-- and restart the probe
--************************************************************************************************

function configureNiSBridge()
	print("Setting NiS Bridge and restarting the secondary NAS probe")
	if nis_bridge_activate ~= nil then
		pdsArgs = pds.create()
		pds.putString(pdsArgs,"name","nas")
		pds.putString(pdsArgs,"section","/setup")
		pds.putString(pdsArgs,"key","nis_bridge")
		pds.putString(pdsArgs,"value",nis_bridge_activate) 
		local count = 1
		repeat
			result,rc = nimbus.request(SecHubAddr..SecHub.."/controller","probe_config_set",pdsArgs,60)
			print("Attempt "..count..": Nimbus Request - Probe Config Set(nis_bridge) for "..SecHubAddr..SecHub.."/controller nas setting key to "..nis_bridge_activate)
			if rc == 0 then
   				print("Setting secondary nas configuration successfull....")
   			else
   				sleep(2000) -- 2 second delay between retries
   				count = count + 1
   			end
   		until rc == 0 or count >=4
   	
		if rc ~= 0 then
   			print("Nimbus Request Error - Probe Config Set(nis_bridge) for "..SecHubAddr..SecHub.."/controller nas (error "..rc..")")
   			nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(nis_bridge) for nas (error "..rc..")")
   		else
   			print("Check nis_bridge configuration setting")
   			cfgArgs = pds.create()
			pds.putString(cfgArgs,"name","nas")
			cfg,rc = nimbus.request(SecHubAddr..SecHub.."/controller","probe_config_get",cfgArgs)
			if rc ~= 0 then
   				print("Nimbus Request Error - Probe Config Get(nas config) failed (error "..rc..")")
   				nimbus.alarm(NIML_MINOR,"Nimbus Request Error - Probe Config Set(nis_bridge) for nas (error "..rc..")")
			else
   				nas = cfg["/setup"]
      			if nas ~= nil then
         			printf("NiS Bridge configuration is now = %s",nas.nis_bridge)
      			else
         			print("Problem getting nas config to verify successful nis_bridge setting")
      			end
			end
			pds.delete(cfgArgs)
		end
		pds.delete (pdsArgs)
	else
		print("Nimbus Request - Probe Config Set(nis_bridge) nis_bridge_activate is nil")
	end
	-- Restart the secondary nas
	print("Restarting "..SecHubAddr..SecHub.."/nas")
	nimbus.request(SecHubAddr..SecHub.."/nas","_restart")
end




--************************************************************************************************
--  Main section
--************************************************************************************************

if mode == "FAILOVER" then
   for _,ump in pairs (umpname) do
      rc = check_umprobot(SecHubAddr,ump,robot_retries)
      if rc then
		print("Found UMP robot "..ump.." - Reconfiguring now")
--4.09      	reconfigure_dap(SecHubAddr,SecHub,ump)
--4.09      	reconfigure_dashboard_engine(SecHubAddr,SecHub,ump)
      	reconfigure_wasp(SecHubAddr,SecHub,ump,SecIP)
      	restart = restart_ump_probes(SecHubAddr,ump)
      	if restart ~= 0 then
      		print("Encountered a problem restarting the UMP probes on "..SecHubAddr..ump)
      		nimbus.alarm(NIML_WARNING,"Encountered a problem restarting the UMP probes on "..SecHubAddr..ump,"probe_failover_restart")
      	end
      else
		print("Ump robot check failed - Aborting reconfiguration")
		nimbus.alarm(NIML_WARNING,"Ump robot check failed - Aborting reconfiguration","failover_robot_check")
      end
   end
   -- Restart the service_host probe on the Secondary hub - 7/29/13
   -- Changed this section from using the _restart command on the service_host probe to having the controller deactivate and activate the probe - 8/02/13
   --
	print("Deactivating "..SecHubAddr..SecHub.."/service_host")
	servArgs = pds.create()
	pds.putString(servArgs,"name","service_host")
	restart, rc = nimbus.request(SecHubAddr..SecHub.."/controller","probe_deactivate",servArgs)
	if rc ~= 0 then
      	print("Encountered a problem deactivating the service_host probe on "..SecHubAddr..SecHub)
      	nimbus.alarm(NIML_WARNING,"Encountered a problem deactivating the service_host probe on "..SecHubAddr..SecHub,"service_host_probe_failover_restart")
    end
    sleep(2000) -- 2 second delay before activating
    print("Activating "..SecHubAddr..SecHub.."/service_host")
	restart, rc = nimbus.request(SecHubAddr..SecHub.."/controller","probe_activate",servArgs)
	if rc ~= 0 then
      	print("Encountered a problem activating the service_host probe on "..SecHubAddr..SecHub)
      	nimbus.alarm(NIML_WARNING,"Encountered a problem activating the service_host probe on "..SecHubAddr..SecHub,"service_host_probe_failover_restart")
    end
    pds.delete(servArgs)
    -- End of added code for the service_host probe restart
    
-- Now check and see if all the ump probes on all the ump robots have started up
	for _,ump in pairs (umpname) do

	rc = check_probe(SecHubAddr,ump,"wasp",wasp_retries)
		if rc then
			print("Wasp probe activation complete on "..SecHubAddr..ump)
		else
			print("Unable to verify wasp probe activation on "..SecHubAddr..ump)
			nimbus.alarm(NIML_WARNING,"Unable to verify wasp probe activation on "..SecHubAddr..ump,"failover_probe_check")
		end
	end
	
elseif mode == "FAILBACK" then
   for _,ump in pairs (umpname) do
		rc = check_umprobot(PriHubAddr,ump,robot_retries)
      	if rc then
			print("Found UMP robot "..ump.." - Reconfiguring now")
      		reconfigure_wasp(PriHubAddr,PriHub,ump,PriIP)
      		restart = restart_ump_probes(PriHubAddr,ump)
      		if restart ~= 0 then
      			print("Encountered a problem restarting the UMP probes on "..PriHubAddr..ump)
      			nimbus.alarm(NIML_WARNING,"Encountered a problem restarting the UMP probes on "..PriHubAddr..ump,"probe_failback_restart")
      		end
		else
			print("Ump robot check failed - Aborting reconfiguration")
			nimbus.alarm(NIML_WARNING,"Ump robot check failed - Aborting reconfiguration","failback_robot_check")
      	end
	end 
	
	-- Now check and see if all the ump probes on all the ump robots have started up
	for _,ump in pairs (umpname) do    	
      	
		rc = check_probe(PriHubAddr,ump,"wasp",wasp_retries)
		if rc then
			print("Wasp probe activation complete on "..PriHubAddr..ump)
		else
			print("Unable to verify wasp probe activation on "..PriHubAddr..ump)
			nimbus.alarm(NIML_WARNING,"Unable to verify wasp probe activation on "..PriHubAddr..ump,"failback_probe_check")
		end
	end
end

--************************************************************************************************
--  Failover complete
--************************************************************************************************
   if mode ~= nil then
         print("UMP "..mode.." COMPLETED")
         sleep(5000)
         nimbus.alarm(NIML_INFORMATION,"UMP "..mode.." COMPLETED")
   else
         print(" UMP_FAILOVER script failed ")
         nimbus.alarm(NIML_MINOR," UMP_FAILOVER script failed ")
   end
   
--************************************************************************************************
--  Closing Alert to prepare for next fail-over
--************************************************************************************************
action.close(a.nimid)

--- Activate/deactivate the NiS Bridge on the secondary nas
-- Uncomment the following line to activate the functionality for the activation/deactivation of the NiS Bridge on the Secondary hub nas probe
configureNiSBridge()


